@@ -29,19 +29,23 @@ sys.path[0:0] = [""]
 
 from nose.plugins.skip import SkipTest
 
+import bson
+from bson import (BSON,
+                  decode_all,
+                  is_valid)
+from bson.binary import Binary
+from bson.code import Code
+from bson.objectid import ObjectId
+from bson.dbref import DBRef
+from bson.son import SON
+from bson.timestamp import Timestamp
+from bson.errors import (InvalidDocument,
+                         InvalidStringData)
+from bson.max_key import MaxKey
+from bson.min_key import MinKey
+from bson.tz_util import (FixedOffset,
+                          utc)
 import pymongo
-from pymongo.binary import Binary
-from pymongo.code import Code
-from pymongo.objectid import ObjectId
-from pymongo.dbref import DBRef
-from pymongo.son import SON
-from pymongo.timestamp import Timestamp
-from pymongo.bson import BSON, is_valid, _to_dicts
-from pymongo.errors import InvalidDocument, InvalidStringData
-from pymongo.max_key import MaxKey
-from pymongo.min_key import MinKey
-from pymongo.tz_util import (FixedOffset,
-                             utc)
 import qcheck
 
 class TestBSON(unittest.TestCase):
@@ -68,91 +72,90 @@ class TestBSON(unittest.TestCase):
         qcheck.check_unittest(self, qcheck.isnt(is_valid),
                               qcheck.gen_string(qcheck.gen_range(0, 40)))
 
-    def test_basic_to_dict(self):
+    def test_basic_decode(self):
         self.assertEqual({"test": u"hello world"},
                          BSON("\x1B\x00\x00\x00\x0E\x74\x65\x73\x74\x00\x0C"
                               "\x00\x00\x00\x68\x65\x6C\x6C\x6F\x20\x77\x6F"
-                              "\x72\x6C\x64\x00\x00").to_dict())
+                              "\x72\x6C\x64\x00\x00").decode())
         self.assertEqual([{"test": u"hello world"}, {}],
-                         _to_dicts("\x1B\x00\x00\x00\x0E\x74\x65\x73\x74\x00"
-                                   "\x0C\x00\x00\x00\x68\x65\x6C\x6C\x6F\x20"
-                                   "\x77\x6F\x72\x6C\x64\x00\x00\x05\x00\x00"
-                                   "\x00\x00"))
+                         decode_all("\x1B\x00\x00\x00\x0E\x74\x65\x73\x74\x00"
+                                    "\x0C\x00\x00\x00\x68\x65\x6C\x6C\x6F\x20"
+                                    "\x77\x6F\x72\x6C\x64\x00\x00\x05\x00\x00"
+                                    "\x00\x00"))
 
     def test_data_timestamp(self):
         self.assertEqual({"test": Timestamp(4, 20)},
                          BSON("\x13\x00\x00\x00\x11\x74\x65\x73\x74\x00\x14"
-                              "\x00\x00\x00\x04\x00\x00\x00\x00").to_dict())
+                              "\x00\x00\x00\x04\x00\x00\x00\x00").decode())
 
-    def test_basic_from_dict(self):
-        self.assertRaises(TypeError, BSON.from_dict, 100)
-        self.assertRaises(TypeError, BSON.from_dict, "hello")
-        self.assertRaises(TypeError, BSON.from_dict, None)
-        self.assertRaises(TypeError, BSON.from_dict, [])
+    def test_basic_encode(self):
+        self.assertRaises(TypeError, BSON.encode, 100)
+        self.assertRaises(TypeError, BSON.encode, "hello")
+        self.assertRaises(TypeError, BSON.encode, None)
+        self.assertRaises(TypeError, BSON.encode, [])
 
-        self.assertEqual(BSON.from_dict({}), BSON("\x05\x00\x00\x00\x00"))
-        self.assertEqual(BSON.from_dict({"test": u"hello world"}),
+        self.assertEqual(BSON.encode({}), BSON("\x05\x00\x00\x00\x00"))
+        self.assertEqual(BSON.encode({"test": u"hello world"}),
                          "\x1B\x00\x00\x00\x02\x74\x65\x73\x74\x00\x0C\x00\x00"
                          "\x00\x68\x65\x6C\x6C\x6F\x20\x77\x6F\x72\x6C\x64\x00"
                          "\x00")
-        self.assertEqual(BSON.from_dict({u"mike": 100}),
+        self.assertEqual(BSON.encode({u"mike": 100}),
                          "\x0F\x00\x00\x00\x10\x6D\x69\x6B\x65\x00\x64\x00\x00"
                          "\x00\x00")
-        self.assertEqual(BSON.from_dict({"hello": 1.5}),
+        self.assertEqual(BSON.encode({"hello": 1.5}),
                          "\x14\x00\x00\x00\x01\x68\x65\x6C\x6C\x6F\x00\x00\x00"
                          "\x00\x00\x00\x00\xF8\x3F\x00")
-        self.assertEqual(BSON.from_dict({"true": True}),
+        self.assertEqual(BSON.encode({"true": True}),
                          "\x0C\x00\x00\x00\x08\x74\x72\x75\x65\x00\x01\x00")
-        self.assertEqual(BSON.from_dict({"false": False}),
+        self.assertEqual(BSON.encode({"false": False}),
                          "\x0D\x00\x00\x00\x08\x66\x61\x6C\x73\x65\x00\x00"
                          "\x00")
-        self.assertEqual(BSON.from_dict({"empty": []}),
+        self.assertEqual(BSON.encode({"empty": []}),
                          "\x11\x00\x00\x00\x04\x65\x6D\x70\x74\x79\x00\x05\x00"
                          "\x00\x00\x00\x00")
-        self.assertEqual(BSON.from_dict({"none": {}}),
+        self.assertEqual(BSON.encode({"none": {}}),
                          "\x10\x00\x00\x00\x03\x6E\x6F\x6E\x65\x00\x05\x00\x00"
                          "\x00\x00\x00")
-        self.assertEqual(BSON.from_dict({"test": Binary("test", 0)}),
+        self.assertEqual(BSON.encode({"test": Binary("test", 0)}),
                          "\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00\x00"
                          "\x00\x00\x74\x65\x73\x74\x00")
-        self.assertEqual(BSON.from_dict({"test": Binary("test")}),
+        self.assertEqual(BSON.encode({"test": Binary("test")}),
                          "\x18\x00\x00\x00\x05\x74\x65\x73\x74\x00\x08\x00\x00"
                          "\x00\x02\x04\x00\x00\x00\x74\x65\x73\x74\x00")
-        self.assertEqual(BSON.from_dict({"test": Binary("test", 128)}),
+        self.assertEqual(BSON.encode({"test": Binary("test", 128)}),
                          "\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00\x00"
                          "\x00\x80\x74\x65\x73\x74\x00")
-        self.assertEqual(BSON.from_dict({"test": None}),
+        self.assertEqual(BSON.encode({"test": None}),
                          "\x0B\x00\x00\x00\x0A\x74\x65\x73\x74\x00\x00")
-        self.assertEqual(BSON.from_dict({"date": datetime.datetime(2007, 1, 8,
-                                                                   0, 30,
-                                                                   11)}),
+        self.assertEqual(BSON.encode({"date": datetime.datetime(2007, 1, 8,
+                                                                0, 30, 11)}),
                          "\x13\x00\x00\x00\x09\x64\x61\x74\x65\x00\x38\xBE\x1C"
                          "\xFF\x0F\x01\x00\x00\x00")
-        self.assertEqual(BSON.from_dict({"regex": re.compile("a*b",
-                                                             re.IGNORECASE)}),
+        self.assertEqual(BSON.encode({"regex": re.compile("a*b",
+                                                          re.IGNORECASE)}),
                          "\x12\x00\x00\x00\x0B\x72\x65\x67\x65\x78\x00\x61\x2A"
                          "\x62\x00\x69\x00\x00")
-        self.assertEqual(BSON.from_dict({"$where": Code("test")}),
+        self.assertEqual(BSON.encode({"$where": Code("test")}),
                          "\x1F\x00\x00\x00\x0F\x24\x77\x68\x65\x72\x65\x00\x12"
                          "\x00\x00\x00\x05\x00\x00\x00\x74\x65\x73\x74\x00\x05"
                          "\x00\x00\x00\x00\x00")
         a = ObjectId("\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B")
-        self.assertEqual(BSON.from_dict({"oid": a}),
+        self.assertEqual(BSON.encode({"oid": a}),
                          "\x16\x00\x00\x00\x07\x6F\x69\x64\x00\x00\x01\x02\x03"
                          "\x04\x05\x06\x07\x08\x09\x0A\x0B\x00")
-        self.assertEqual(BSON.from_dict({"ref": DBRef("coll", a)}),
+        self.assertEqual(BSON.encode({"ref": DBRef("coll", a)}),
                          "\x2F\x00\x00\x00\x03ref\x00\x25\x00\x00\x00\x02$ref"
                          "\x00\x05\x00\x00\x00coll\x00\x07$id\x00\x00\x01\x02"
                          "\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x00\x00")
 
-    def test_from_then_to_dict(self):
+    def test_encode_then_decode(self):
 
         def helper(dict):
-            self.assertEqual(dict, (BSON.from_dict(dict)).to_dict())
+            self.assertEqual(dict, (BSON.encode(dict)).decode())
         helper({})
         helper({"test": u"hello"})
-        self.assert_(isinstance(BSON.from_dict({"hello": "world"})
-                                .to_dict()["hello"],
+        self.assert_(isinstance(BSON.encode({"hello": "world"})
+                                .decode()["hello"],
                                 unicode))
         helper({"mike": -10120})
         helper({"long": long(10)})
@@ -176,17 +179,17 @@ class TestBSON(unittest.TestCase):
         helper({"foo": MinKey()})
         helper({"foo": MaxKey()})
 
-        def from_then_to_dict(dict):
-            return dict == (BSON.from_dict(dict)).to_dict()
+        def encode_then_decode(dict):
+            return dict == (BSON.encode(dict)).decode()
 
-        qcheck.check_unittest(self, from_then_to_dict,
+        qcheck.check_unittest(self, encode_then_decode,
                               qcheck.gen_mongo_dict(3))
 
     def test_aware_datetime(self):
         aware = datetime.datetime(1993, 4, 4, 2, tzinfo=FixedOffset(555, "SomeZone"))
         as_utc = (aware - aware.utcoffset()).replace(tzinfo=utc)
         self.assertEqual(datetime.datetime(1993, 4, 3, 16, 45, tzinfo=utc), as_utc)
-        after = BSON.from_dict({"date": aware}).to_dict(tz_aware=True)["date"]
+        after = BSON.encode({"date": aware}).decode(tz_aware=True)["date"]
         self.assertEqual(utc, after.tzinfo)
         self.assertEqual(as_utc, after)
 
@@ -194,35 +197,35 @@ class TestBSON(unittest.TestCase):
         aware = datetime.datetime(1993, 4, 4, 2, tzinfo=FixedOffset(555, "SomeZone"))
         naive_utc = (aware - aware.utcoffset()).replace(tzinfo=None)
         self.assertEqual(datetime.datetime(1993, 4, 3, 16, 45), naive_utc)
-        after = BSON.from_dict({"date": aware}).to_dict()["date"]
+        after = BSON.encode({"date": aware}).decode()["date"]
         self.assertEqual(None, after.tzinfo)
         self.assertEqual(naive_utc, after)
 
     def test_dst(self):
         d = {"x": datetime.datetime(1993, 4, 4, 2)}
-        self.assertEqual(d, BSON.from_dict(d).to_dict())
+        self.assertEqual(d, BSON.encode(d).decode())
 
     def test_bad_encode(self):
-        self.assertRaises(InvalidStringData, BSON.from_dict,
+        self.assertRaises(InvalidStringData, BSON.encode,
                           {"lalala": '\xf4\xe0\xf0\xe1\xc0 Color Touch'})
 
     def test_overflow(self):
-        self.assert_(BSON.from_dict({"x": 9223372036854775807L}))
-        self.assertRaises(OverflowError, BSON.from_dict, {"x": 9223372036854775808L})
+        self.assert_(BSON.encode({"x": 9223372036854775807L}))
+        self.assertRaises(OverflowError, BSON.encode, {"x": 9223372036854775808L})
 
-        self.assert_(BSON.from_dict({"x": -9223372036854775808L}))
-        self.assertRaises(OverflowError, BSON.from_dict, {"x": -9223372036854775809L})
+        self.assert_(BSON.encode({"x": -9223372036854775808L}))
+        self.assertRaises(OverflowError, BSON.encode, {"x": -9223372036854775809L})
 
     def test_tuple(self):
         self.assertEqual({"tuple": [1, 2]},
-                          BSON.from_dict({"tuple": (1, 2)}).to_dict())
+                          BSON.encode({"tuple": (1, 2)}).decode())
 
     def test_uuid(self):
         if not should_test_uuid:
             raise SkipTest()
 
         id = uuid.uuid4()
-        transformed_id = (BSON.from_dict({"id": id})).to_dict()["id"]
+        transformed_id = (BSON.encode({"id": id})).decode()["id"]
 
         self.assert_(isinstance(transformed_id, uuid.UUID))
         self.assertEqual(id, transformed_id)
@@ -232,74 +235,74 @@ class TestBSON(unittest.TestCase):
     # that doesn't really test anything but the lack of a segfault.
     def test_unicode_regex(self):
         regex = re.compile(u'revisi\xf3n')
-        BSON.from_dict({"regex": regex}).to_dict()
+        BSON.encode({"regex": regex}).decode()
 
     def test_non_string_keys(self):
-        self.assertRaises(InvalidDocument, BSON.from_dict, {8.9: "test"})
+        self.assertRaises(InvalidDocument, BSON.encode, {8.9: "test"})
 
     def test_large_document(self):
-        self.assertRaises(InvalidDocument, BSON.from_dict, {"key": "x"*4*1024*1024})
+        self.assertRaises(InvalidDocument, BSON.encode, {"key": "x"*4*1024*1024})
 
     def test_utf8(self):
         w = {u"aéあ": u"aéあ"}
-        self.assertEqual(w, BSON.from_dict(w).to_dict())
+        self.assertEqual(w, BSON.encode(w).decode())
 
         x = {u"aéあ".encode("utf-8"): u"aéあ".encode("utf-8")}
-        self.assertEqual(w, BSON.from_dict(x).to_dict())
+        self.assertEqual(w, BSON.encode(x).decode())
 
         y = {"hello": u"aé".encode("iso-8859-1")}
-        self.assertRaises(InvalidStringData, BSON.from_dict, y)
+        self.assertRaises(InvalidStringData, BSON.encode, y)
 
         z = {u"aé".encode("iso-8859-1"): "hello"}
-        self.assertRaises(InvalidStringData, BSON.from_dict, z)
+        self.assertRaises(InvalidStringData, BSON.encode, z)
 
     def test_null_character(self):
         doc = {"a": "\x00"}
-        self.assertEqual(doc, BSON.from_dict(doc).to_dict())
+        self.assertEqual(doc, BSON.encode(doc).decode())
 
         doc = {"a": u"\x00"}
-        self.assertEqual(doc, BSON.from_dict(doc).to_dict())
+        self.assertEqual(doc, BSON.encode(doc).decode())
 
-        self.assertRaises(InvalidDocument, BSON.from_dict, {"\x00": "a"})
-        self.assertRaises(InvalidDocument, BSON.from_dict, {u"\x00": "a"})
+        self.assertRaises(InvalidDocument, BSON.encode, {"\x00": "a"})
+        self.assertRaises(InvalidDocument, BSON.encode, {u"\x00": "a"})
 
-        self.assertRaises(InvalidDocument, BSON.from_dict, {"a": re.compile("ab\x00c")})
-        self.assertRaises(InvalidDocument, BSON.from_dict, {"a": re.compile(u"ab\x00c")})
+        self.assertRaises(InvalidDocument, BSON.encode, {"a": re.compile("ab\x00c")})
+        self.assertRaises(InvalidDocument, BSON.encode, {"a": re.compile(u"ab\x00c")})
 
     def test_move_id(self):
         self.assertEqual("\x19\x00\x00\x00\x02_id\x00\x02\x00\x00\x00a\x00"
                          "\x02a\x00\x02\x00\x00\x00a\x00\x00",
-                         BSON.from_dict(SON([("a", "a"), ("_id", "a")])))
+                         BSON.encode(SON([("a", "a"), ("_id", "a")])))
 
         self.assertEqual("\x2c\x00\x00\x00"
                          "\x02_id\x00\x02\x00\x00\x00b\x00"
                          "\x03b\x00"
                          "\x19\x00\x00\x00\x02a\x00\x02\x00\x00\x00a\x00"
                          "\x02_id\x00\x02\x00\x00\x00a\x00\x00\x00",
-                         BSON.from_dict(SON([("b", SON([("a", "a"), ("_id", "a")])),
-                                             ("_id", "b")])))
+                         BSON.encode(SON([("b", SON([("a", "a"), ("_id", "a")])),
+                                          ("_id", "b")])))
 
     def test_dates(self):
         doc = {"early": datetime.datetime(1686, 5, 5),
                "late": datetime.datetime(2086, 5, 5)}
         try:
-            self.assertEqual(doc, BSON.from_dict(doc).to_dict())
+            self.assertEqual(doc, BSON.encode(doc).decode())
         except ValueError:
             # Ignore ValueError when no C ext, since it's probably
             # a problem w/ 32-bit Python - we work around this in the
             # C ext, though.
-            if pymongo.has_c():
+            if bson.has_c():
                 raise
 
     def test_custom_class(self):
-        self.assert_(isinstance(BSON.from_dict({}).to_dict(), dict))
-        self.assertFalse(isinstance(BSON.from_dict({}).to_dict(), SON))
-        self.assert_(isinstance(BSON.from_dict({}).to_dict(SON), SON))
+        self.assert_(isinstance(BSON.encode({}).decode(), dict))
+        self.assertFalse(isinstance(BSON.encode({}).decode(), SON))
+        self.assert_(isinstance(BSON.encode({}).decode(SON), SON))
 
-        self.assertEqual(1, BSON.from_dict({"x": 1}).to_dict(SON)["x"])
+        self.assertEqual(1, BSON.encode({"x": 1}).decode(SON)["x"])
 
-        x = BSON.from_dict({"x": [{"y": 1}]})
-        self.assert_(isinstance(x.to_dict(SON)["x"][0], SON))
+        x = BSON.encode({"x": [{"y": 1}]})
+        self.assert_(isinstance(x.decode(SON)["x"][0], SON))
 
     def test_subclasses(self):
         # make sure we can serialize subclasses of native Python types.
@@ -312,13 +315,21 @@ class TestBSON(unittest.TestCase):
         d = {'a' : _myint(42), 'b' : _myfloat(63.9),
              'c' : _myunicode('hello world')
             }
-        d2 = BSON.from_dict(d).to_dict()
+        d2 = BSON.encode(d).decode()
         for key, value in d2.iteritems():
             orig_value = d[key]
             orig_type = orig_value.__class__.__bases__[0]
             self.assertEqual(type(value), orig_type)
             self.assertEqual(value, orig_type(value))
 
+    def test_ordered_dict(self):
+        try:
+            from collections import OrderedDict
+        except ImportError:
+            raise SkipTest()
+        d = OrderedDict([("one", 1), ("two", 2), ("three", 3), ("four", 4)])
+        self.assertEqual(d, BSON.encode(d).decode(as_class=OrderedDict))
+
 
 if __name__ == "__main__":
     unittest.main()
