@@ -27,9 +27,11 @@ from nose.plugins.skip import SkipTest
 
 sys.path[0:0] = [""]
 
+from bson.binary import Binary
+from bson.code import Code
+from bson.objectid import ObjectId
+from bson.son import SON
 from pymongo import ASCENDING, DESCENDING
-from pymongo.binary import Binary
-from pymongo.code import Code
 from pymongo.collection import Collection
 from pymongo.errors import (DuplicateKeyError,
                             InvalidDocument,
@@ -37,8 +39,6 @@ from pymongo.errors import (DuplicateKeyError,
                             InvalidOperation,
                             OperationFailure,
                             TimeoutError)
-from pymongo.objectid import ObjectId
-from pymongo.son import SON
 from test.test_connection import get_connection
 from test import (qcheck,
                   version)
@@ -102,22 +102,17 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"})
                          .count(), 1)
         db.test.create_index("hello")
-        self.assert_(SON([(u"name", u"hello_1"),
-                          (u"ns", u"pymongo_test.test"),
-                          (u"key", SON([(u"hello", 1)]))]) in
-                     list(db.system.indexes
-                          .find({"ns": u"pymongo_test.test"})))
+        self.assert_(u"hello_1" in
+                     [a["name"] for a in db.system.indexes
+                      .find({"ns": u"pymongo_test.test"})])
 
         db.test.drop_indexes()
         self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"})
                          .count(), 1)
         db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)])
-        self.assert_(SON([(u"name", u"hello_-1_world_1"),
-                          (u"ns", u"pymongo_test.test"),
-                          (u"key", SON([(u"hello", -1),
-                                        (u"world", 1)]))]) in
-                     list(db.system.indexes
-                          .find({"ns": u"pymongo_test.test"})))
+        self.assert_(u"hello_-1_world_1" in
+                     [a["name"] for a in db.system.indexes
+                      .find({"ns": u"pymongo_test.test"})])
 
     def test_ensure_index(self):
         db = self.db
@@ -152,12 +147,6 @@ class TestCollection(unittest.TestCase):
                          db.test.ensure_index("goodbye"))
         self.assertEqual(None, db.test.ensure_index("goodbye"))
 
-        db_name = self.db.name
-        self.connection.drop_database(self.db.name)
-        self.assertEqual("goodbye_1",
-                         db.test.ensure_index("goodbye"))
-        self.assertEqual(None, db.test.ensure_index("goodbye"))
-
         db.test.drop_index("goodbye_1")
         self.assertEqual("goodbye_1",
                          db.test.create_index("goodbye"))
@@ -203,11 +192,9 @@ class TestCollection(unittest.TestCase):
         db.test.drop_index(name)
         self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"})
                          .count(), 2)
-        self.assert_(SON([(u"name", u"hello_1"),
-                          (u"ns", u"pymongo_test.test"),
-                          (u"key", SON([(u"hello", 1)]))]) in
-                     list(db.system.indexes
-                          .find({"ns": u"pymongo_test.test"})))
+        self.assert_(u"hello_1" in
+                     [a["name"] for a in db.system.indexes
+                      .find({"ns": u"pymongo_test.test"})])
 
         db.test.drop_indexes()
         db.test.create_index("hello")
@@ -219,11 +206,9 @@ class TestCollection(unittest.TestCase):
         db.test.drop_index([("goodbye", ASCENDING)])
         self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"})
                          .count(), 2)
-        self.assert_(SON([(u"name", u"hello_1"),
-                          (u"ns", u"pymongo_test.test"),
-                          (u"key", SON([(u"hello", 1)]))]) in
-                     list(db.system.indexes
-                          .find({"ns": u"pymongo_test.test"})))
+        self.assert_(u"hello_1" in
+                     [a["name"] for a in db.system.indexes
+                      .find({"ns": u"pymongo_test.test"})])
 
     def test_index_info(self):
         db = self.db
@@ -235,17 +220,17 @@ class TestCollection(unittest.TestCase):
 
         db.test.create_index("hello")
         self.assertEqual(len(db.test.index_information()), 2)
-        self.assertEqual(db.test.index_information()["hello_1"],
-                         {"key": [("hello", ASCENDING)]})
+        self.assertEqual(db.test.index_information()["hello_1"]["key"],
+                         [("hello", ASCENDING)])
 
         db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)],
                              unique=True)
-        self.assertEqual(db.test.index_information()["hello_1"],
-                         {"key": [("hello", ASCENDING)]})
+        self.assertEqual(db.test.index_information()["hello_1"]["key"],
+                         [("hello", ASCENDING)])
         self.assertEqual(len(db.test.index_information()), 3)
-        self.assertEqual({"key": [("hello", DESCENDING), ("world", ASCENDING)],
-                          "unique": True},
-                         db.test.index_information()["hello_-1_world_1"])
+        self.assertEqual([("hello", DESCENDING), ("world", ASCENDING)],
+                         db.test.index_information()["hello_-1_world_1"]["key"])
+        self.assertEqual(True, db.test.index_information()["hello_-1_world_1"]["unique"])
 
     def test_field_selection(self):
         db = self.db
@@ -406,24 +391,24 @@ class TestCollection(unittest.TestCase):
         db.test.insert({"hello": "world"})
         db.test.insert({"hello": {"hello": "world"}})
 
-        self.assertRaises(InvalidName, db.test.insert, {"$hello": "world"})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert, {"$hello": "world"})
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hello": {"$hello": "world"}})
 
         db.test.insert({"he$llo": "world"})
         db.test.insert({"hello": {"hello$": "world"}})
 
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {".hello": "world"})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hello": {".hello": "world"}})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hello.": "world"})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hello": {"hello.": "world"}})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hel.lo": "world"})
-        self.assertRaises(InvalidName, db.test.insert,
+        self.assertRaises(InvalidDocument, db.test.insert,
                           {"hello": {"hel.lo": "world"}})
 
         db.test.update({"hello": "world"}, {"$inc": "hello"})
@@ -663,6 +648,11 @@ class TestCollection(unittest.TestCase):
         self.db.test.remove({"x": 1}, w=1, wtimeout=1)
         self.db.test.update({"x": 1}, {"y": 2}, w=1, wtimeout=1)
 
+    def test_manual_last_error(self):
+        self.db.test.save({"x": 1})
+        self.assertRaises(TimeoutError, self.db.command, "getlasterror", w=2, wtimeout=1)
+        self.db.command("getlasterror", w=1, wtimeout=1)
+
     def test_count(self):
         db = self.db
         db.drop_collection("test")
@@ -1058,7 +1048,7 @@ class TestCollection(unittest.TestCase):
 
     def test_as_class(self):
         c = self.db.test
-        c.remove()
+        c.drop()
         c.insert({"x": 1})
 
         self.assert_(isinstance(c.find().next(), dict))
