@@ -24,6 +24,8 @@ sys.path[0:0] = [""]
 
 from nose.plugins.skip import SkipTest
 
+from bson.son import SON
+from bson.tz_util import utc
 from pymongo.connection import (Connection,
                                 _parse_uri)
 from pymongo.database import Database
@@ -33,8 +35,6 @@ from pymongo.errors import (AutoReconnect,
                             InvalidName,
                             InvalidURI,
                             OperationFailure)
-from pymongo.son import SON
-from pymongo.tz_util import utc
 from test import version
 
 
@@ -149,7 +149,7 @@ class TestConnection(unittest.TestCase):
 
         self.assertRaises(InvalidName, c.copy_database, "foo", "$foo")
 
-        c.drop_database("pymongo_test")
+        c.pymongo_test.test.drop()
         c.drop_database("pymongo_test1")
         c.drop_database("pymongo_test2")
 
@@ -169,10 +169,9 @@ class TestConnection(unittest.TestCase):
         self.assert_("pymongo_test2" in c.database_names())
         self.assertEqual("bar", c.pymongo_test2.test.find_one()["foo"])
 
-        c.drop_database("pymongo_test1")
-        c.drop_database("pymongo_test2")
-
         if version.at_least(c, (1, 3, 3, 1)):
+            c.drop_database("pymongo_test1")
+
             c.pymongo_test.add_user("mike", "password")
 
             self.assertRaises(OperationFailure, c.copy_database,
@@ -190,8 +189,6 @@ class TestConnection(unittest.TestCase):
             self.assert_("pymongo_test1" in c.database_names())
             self.assertEqual("bar", c.pymongo_test1.test.find_one()["foo"])
 
-            c.drop_database("pymongo_test1")
-
     def test_iteration(self):
         connection = Connection(self.host, self.port)
 
