@@ -20,24 +20,25 @@ import sys
 sys.path[0:0] = [""]
 import unittest
 
+from bson.code import Code
+from bson.dbref import DBRef
+from bson.objectid import ObjectId
+from bson.son import SON
 from pymongo import (ALL,
                      ASCENDING,
                      DESCENDING,
                      helpers,
                      OFF,
                      SLOW_ONLY)
-from pymongo.code import Code
 from pymongo.collection import Collection
 from pymongo.connection import Connection
 from pymongo.database import Database
-from pymongo.dbref import DBRef
 from pymongo.errors import (CollectionInvalid,
                             InvalidName,
                             InvalidOperation,
                             OperationFailure)
-from pymongo.objectid import ObjectId
-from pymongo.son import SON
-from pymongo.son_manipulator import AutoReference, NamespaceInjector
+from pymongo.son_manipulator import (AutoReference,
+                                     NamespaceInjector)
 from test import version
 from test.test_connection import get_connection
 
@@ -496,6 +497,20 @@ class TestDatabase(unittest.TestCase):
         db.system_js.no_param = Code("return 5;")
         self.assertEqual(5, db.system_js.no_param())
 
+    def test_system_js_list(self):
+        db = self.connection.pymongo_test
+        db.system.js.remove()
+        self.assertEqual([], db.system_js.list())
+
+        db.system_js.foo = "blah"
+        self.assertEqual(["foo"], db.system_js.list())
+
+        db.system_js.bar = "baz"
+        self.assertEqual(set(["foo", "bar"]), set(db.system_js.list()))
+
+        del db.system_js.foo
+        self.assertEqual(["bar"], db.system_js.list())
+
 
 if __name__ == "__main__":
     unittest.main()
