@@ -26,18 +26,23 @@ except ImportError:
         import simplejson as json
     except ImportError:
         json_lib = False
+try:
+    import uuid
+    should_test_uuid = True
+except ImportError:
+    should_test_uuid = False
 
 from nose.plugins.skip import SkipTest
 
 sys.path[0:0] = [""]
 
-from pymongo.json_util import default, object_hook
-from pymongo.objectid import ObjectId
-from pymongo.dbref import DBRef
-from pymongo.min_key import MinKey
-from pymongo.max_key import MaxKey
-from pymongo.timestamp import Timestamp
-from pymongo.tz_util import utc
+from bson.objectid import ObjectId
+from bson.dbref import DBRef
+from bson.min_key import MinKey
+from bson.max_key import MaxKey
+from bson.timestamp import Timestamp
+from bson.tz_util import utc
+from bson.json_util import default, object_hook
 
 class TestJsonUtil(unittest.TestCase):
 
@@ -61,10 +66,15 @@ class TestJsonUtil(unittest.TestCase):
     def test_dbref(self):
         self.round_trip({"ref": DBRef("foo", 5)})
         self.round_trip({"ref": DBRef("foo", 5, "db")})
-        self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5}}",
-                         json.dumps({"ref": DBRef("foo", 5)}, default=default))
-        self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5, \"$db\": \"bar\"}}",
-                         json.dumps({"ref": DBRef("foo", 5, "bar")}, default=default))
+
+        # TODO this is broken when using cjson. See:
+        #   http://jira.mongodb.org/browse/PYTHON-153
+        #   http://bugs.python.org/issue6105
+        #
+        # self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5}}",
+        #                  json.dumps({"ref": DBRef("foo", 5)}, default=default))
+        # self.assertEqual("{\"ref\": {\"$ref\": \"foo\", \"$id\": 5, \"$db\": \"bar\"}}",
+        #                  json.dumps({"ref": DBRef("foo", 5, "bar")}, default=default))
 
     def test_datetime(self):
         # only millis, not micros
@@ -88,5 +98,10 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(dct['ts']['t'], 4)
         self.assertEqual(dct['ts']['i'], 13)
 
+    def test_uuid(self):
+        if not should_test_uuid:
+            raise SkipTest()
+        self.round_trip({'uuid' : uuid.UUID('f47ac10b-58cc-4372-a567-0e02b2c3d479')})
+
 if __name__ == "__main__":
     unittest.main()
